/****************************************************************************
*
*    Planning scheduler for CAN
*
*          CAN station v9
*
*  A master broadcasts periodically (each EC) a
*       master message indicating which variables
*       must be produced thereafter, and the time required for the transmission.
*  Each station must decode that message,
*  Scan a local buffer with all messages to
*       produce by that station,
*  Identify which are to be produced, access
*       the global messages' table  and produce
*       them starting from the right moment.
*  Messages are produced with I/O-CPU overlapping
*  IDs of received messages are indexes to the
*       global messages' table (if station is
*       consumer, consume them, one by one)
*  Tx and rx via DMA (message buffer is in internal memory)
*  Empty slots within global messages's table is signalled
*       with a null status field.
*  A station can have up to MAX_VAR variables all together but
*       only up to STA_MAX_VAR to be produced
*  Rest equal to version 6
*
*  Works together with "master9.c" and "plansch9.h".
*
****************************************************************************/

#define	STATION1

#define	TEST_ASY

/*****************************/
/***** MITs for asynch mesgs */

				/* Station 1 */
#define	AMS1 2	/* Asynchronous message mit 	*/
#define AIS1 12 /* Asynchronous message ID 		*/
#define ALS1 1  /* Asynchronous message length 	*/

#define	AMS2 2
#define AIS2 11
#define ALS2 5

#define	AMS3 1
#define AIS3 10
#define ALS3 7

#define	AMS4 1
#define AIS4 9
#define ALS4 4

#define	AMS5 1
#define AIS5 8
#define ALS5 5

#define	AMS6 1
#define AIS6 7
#define ALS6 4

/*
#define	AMS1 1
#define AIS1 7
#define ALS1 4

#define	AMS2 1
#define AIS2 8
#define ALS2 5

#define	AMS3 1
#define AIS3 9
#define ALS3 4

#define	AMS4 1
#define AIS4 10
#define ALS4 7

#define	AMS5 1
#define AIS5 11
#define ALS5 5

#define	AMS6 1
#define AIS6 12
#define ALS6 1
*/


#include "io592.h"
#include "80592.h" 
#include "plansch8.h"
#include <special.h>

/***********************/
/* Function prototypes */
/***********************/
void ini_globals(void); 
void ini_CAN(void);
void ini_T2(void);
void AM_ini(void);

signed char StaTab_AddVar(StaVarType*);
signed char VarStatus(unsigned char);
signed char produce(unsigned char, unsigned char *);
signed char consume(unsigned char, unsigned char *);

signed char AM_StaTab_AddVar(AM_ConsStaVarType* );
signed char AM_VarStatus(unsigned char );
signed char AM_consume (unsigned char, unsigned char *);
signed char AM_produce(unsigned char, unsigned char, unsigned char *);

signed char AM_getbuf(/* unsigned char, unsigned char *, unsigned char * */);


/****************/
/* Global vars: */
/***************/
data unsigned char IntMesgBuf[10];  /* internal message buffer (full frame) for Rx */
data unsigned char IntMMBuf[8];     /* internal buffer for master message data, only */
data unsigned char IntTxBuf[10];    /* internal message buffer (full frame) for Tx */
data unsigned char StProdByteIndex[STA_MAX_VAR];    /* byte_index of mesgs to be produced by this st*/
data unsigned char StProdBitMask[STA_MAX_VAR];      /* bit_mask of mesgs to be produced by this st*/
data unsigned char ScanIndex;   /* used to control production buffer scanning */
data unsigned char StNProdMesg; /* n. of messages produced by this station */
data unsigned char NEcTxMesg;   /* n. of messages sent (in tx buffer) */
data unsigned char NEcRxMesg;   /* used in rx routine to count n. of rx data messages */

data unsigned char smlen_500us; /* Length of synch. messages to produce in # of 500us slots */

data unsigned char AMS_TxReqPend;/* Signals if there is an asynch. mesg Tx pending */

StaVTabType var_table;              /* Table with sync. mesgs properties    */
AM_ConsStaVTabType am_var_table;    /* Table with asynch. mesgs properties  */
                                    /*   which are consumed in this station */

/* Asynchronous message buffer */
idata AM_BufStaVTabType AM_buffer;       /* Buffer for asynchronous message production */

data unsigned char AM_len;      /* Length of one AM buffer entry */
data unsigned char AM_data[10]; /* AM buffer entry (all frame : header + data ) */

data unsigned char StatReg;     /* Status Register copy */

data unsigned char AM_stop;          /* Signal if production of asynch mesgs is allowed */

/* Globals of S1_int */
data unsigned char S1_i, S1_j;

data unsigned char win_phase;     /* Signals if current EC phase is either sync. or async. window */

data unsigned char T0CAN_intinv;  /* Signals if has occurred an inversion on the */
								  /* interrupt order in the beggining of the EC	*/
								  /* The correct sequence is T0int->CAN Rx int	*/

data unsigned char SEC_count;    /* Counter of EC's - used by synchronous messages*/ 
data unsigned char AEC_count;    /* Counter of EC's - used by asynchronous messages*/

/* Local variables and comunication parameters for AM_getbuf */
data unsigned char *AMgb_aux_var;   
data unsigned char AMgb_i;
data unsigned char AMgb_getmode;
data unsigned char AMgb_buflen;
data unsigned char AMgb_data[10];

/* Global vars for time measurement (Timer 2) */
data unsigned char T2B_Ticks;	/* Number of byte overflow of timer T2 (256us each) */


data unsigned char A_T2B_Ticks;	/* Data variables for quick store while ints disabled */
data unsigned char A_TML2;
data unsigned char A_TMH2;

/* General variables*/
data unsigned char vTH0;
data unsigned char Flag_T2;

data unsigned char errcode;

data unsigned char FMeasure;	/* Flag for taking measures */

unsigned int lixo,lixo2;

/* Debug and measurement global variables */
	#define DBGBUFSIZE 1024
	unsigned char dbgbuf[DBGBUFSIZE][5];
	unsigned int VT2B[DBGBUFSIZE];
	unsigned int dbgbufindex;
/* Debug-END */

/**********************************************************
*   main
*
*  initiate globals and CAN controller to generate int
*  upon detection of master message.
*  infinite empty loop.
*  the required processing in done
*  in the CAN and Timer0 interrupt service routines.
*
***********************************************************/
void main(void)
{

    StaVarType aux_var;
    AM_ConsStaVarType am_aux_var;
    signed char vstatus,base,base1,base2,base3,base4;
	unsigned int i;
	data unsigned int T2T_Val;

    /* Variables' values */
    unsigned char value1[8],value2[8],value3[8],value4[8];
    unsigned char value5[8],value6[8],value7[8],value8[8],value9[8];

    signed char result;
    unsigned char mesgid;
    unsigned char mesglen;

	P4.3=0;

    RS232_ini(0xfd,0,0);
    printf("\n\nSta8 beginning....");

    /****************************************************************/
    /* This set of functions must be called to proper system set-up */
    ini_globals();  /* Init global vars and timers */
	ini_CAN();      /* Initialise CAN interface */
	ini_T2();		/* Initialise Timer 2 */
	AM_ini();       /* Initialize AMS system */
	/****************************************************************/

	/******************************/
	/* User codes starts here ... */
	/******************************/

    /*  Build variables' table */
    /* Note: for each variable add, if error set P4.7 if vstatus < 0 */
	P4.7 = 0;


	/* Update Measurement Flag */
	P4.1=1;
	if(P4.0==0)
		FMeasure=1;
	else
		FMeasure=0;


#ifdef STATION1
 	aux_var.vid=0x01;
	aux_var.size=1;
	aux_var.status=0;
	aux_var.status|=PRODUCER;
	vstatus=StaTab_AddVar(&aux_var);
	if( vstatus < 0 )
		P4.7 = 1;

    aux_var.vid=0x05;
	aux_var.size=5;
	aux_var.status=0;
	aux_var.status|=PRODUCER;
	vstatus=StaTab_AddVar(&aux_var);
	if( vstatus < 0 )
		P4.7 = 1;


#endif

#ifdef STATION2

	/* Synchronous Messages */

	aux_var.vid=0x02;
	aux_var.size=3;
	aux_var.status=0;
	aux_var.status|=PRODUCER;
	vstatus=StaTab_AddVar(&aux_var);
	if( vstatus < 0 )
		P4.7 = 1;

	aux_var.vid=0x06;
	aux_var.size=5;
	aux_var.status=0;
	aux_var.status|=PRODUCER;
	vstatus=StaTab_AddVar(&aux_var);
	if( vstatus < 0 )
		P4.7 = 1;
		
	/* Asynchronous Messages (station is a consumer) */
	/* *************  Para teste de consumo de aperiodicas apenas 
	am_aux_var.vid=0x01;
	am_aux_var.size=4;
	vstatus=AM_StaTab_AddVar(&am_aux_var);
   	 if( vstatus < 0 )
		P4.7 = 1;

	am_aux_var.vid=0x02;
   	am_aux_var.size=4;
	vstatus=AM_StaTab_AddVar(&am_aux_var);
   	 if( vstatus < 0 )
		P4.7 = 1;
		
	am_aux_var.vid=0x03;
	am_aux_var.size=4;
   	am_aux_var.status=0;
	vstatus=AM_StaTab_AddVar(&am_aux_var);
	if( vstatus < 0 )
		P4.7 = 1;

	**************** */
#endif

#ifdef STATION3
	aux_var.vid=0x03;
	aux_var.size=3;
	aux_var.status=0;
	aux_var.status|=PRODUCER;
	vstatus=StaTab_AddVar(&aux_var);
	if( vstatus < 0 )
		P4.7 = 1;

#endif
	
#ifdef STATION4
	aux_var.vid=0x04;
	aux_var.size=2;
	aux_var.status=0;
	aux_var.status|=PRODUCER;
	vstatus=StaTab_AddVar(&aux_var);
	if( vstatus < 0 )
		P4.7 = 1;
#endif



	
	base = 0;
	base1 = 1;
	base2 = 2;
	base3=0;
	base4=0;

	dbgbufindex=0;

	SEC_count=0;
	AEC_count=0;
    
	value1[0] = 1;
	value1[1] = 2;
	value1[2] = 3;
	value1[3] = 4;
	value1[4] = 5;
	value1[5] = 6;
 	value1[6] = 7;
	value1[7] = 8;			

	value2[0] = 2;
	value2[1] = 3;
	value2[2] = 4;
	value2[3] = 5;
	value2[4] = 6;
	value2[5] = 7;
	value2[6] = 8;
	value2[7] = 9;			

	value3[0] = 3;
	value3[1] = 4;
	value3[2] = 5;
	value3[3] = 6;
	value3[4] = 7;
	value3[5] = 8;
	value3[6] = 9;
	value3[7] = 10;			
	
	value4[0] = 4;
	value4[1] = 5;
	value4[2] = 6;
	value4[3] = 7;
	value4[4] = 8;
	value4[5] = 9;
	value4[6] = 10;
	value4[7] = 11;			
   
	value5[0] = 5;
	value5[1] = 6;
	value5[2] = 7;
	value5[3] = 8;
	value5[4] = 9;
	value5[5] = 10;
	value5[6] = 11;
	value5[7] = 12;			

	value6[0] = 6;
	value6[1] = 7;
	value6[2] = 8;
	value6[3] = 9;
	value6[4] = 10;
	value6[5] = 11;
	value6[6] = 12;
	value6[7] = 13;			

	value7[0] = 7;
	value7[1] = 8;
	value7[2] = 9;
	value7[3] = 10;
	value7[4] = 11;
	value7[5] = 12;
	value7[6] = 13;
	value7[7] = 14;			

	value8[0] = 8;
	value8[1] = 9;
	value8[2] = 10;
	value8[3] = 11;
	value8[4] = 12;
	value8[5] = 13;
	value8[6] = 14;
	value8[7] = 15;			

	value9[0] = 9;
	value9[1] = 10;
	value9[2] = 11;
	value9[3] = 12;
	value9[4] = 13;
	value9[5] = 14;
	value9[6] = 15;
	value9[7] = 16;			

#ifdef STATION1
	
	base=1;
	base1=5;	
	base2=0;
	base3=7;
	while(1) 	/* **** Infinite cycle  ***** */
	{
		P4.7 = 0;

		/********************************************/
		/* Test for synchronous messages      */
		/********************************************/
		if(SEC_count > 0)
		{

        	/* If entering in a new EC */
			SEC_count = 0;
			base2++;

			value1[0] = base;
			produce( 1,value1);
			base++;

			if(base2>=2)
			{
				value5[0] = base1;
				produce( 5,value5);
				base1++;
				base2=0;
			}
		}

       	#ifdef TEST_ASY
			/********************************************/
			/* Test for asynchronous messages           */
       	 	/********************************************/
			if(AEC_count > AMS1-1)
			{

				AEC_count=0;

				while(AM_stop == FALSE)
				{
					U8051NOP
				}


				/* Produce async mesg AIS1 with a "period" of AMS1 ECs */
				value9[0]=base3;
				mesgid=AIS1;
				mesglen=ALS1;


				result=AM_produce(mesgid,mesglen,value9);
				if(result == AMOK)
				{
					// printf("\nAM_produce AMOK");
				}
				base3++;

			}

			/* If debug mode ... */


			if(dbgbufindex >= DBGBUFSIZE-10)
			{
				EA = 0;

				printf("\n\n Tempos de producao/transmissao de mensagens\n....");
				printf("\n Station 1 : Mesg ID=%d Len=%d mit=%d\n",AIS1,ALS1,AMS1);
				for(i=0;i<DBGBUFSIZE-10;i++)
				{
					printf("\n>>> SN:%d : T2B_Ticks:%d : TMH2:%d : TML2:%d : Tipo:%d",i,dbgbuf[i][0],dbgbuf[i][1],dbgbuf[i][2],dbgbuf[i][3]);
				}
				while(1);
				EA = 1;
				dbgbufindex =0;
			}


		#endif
 	}
#endif

#ifdef STATION2
	base=2;
	base1=6;	
	base2=0;
	base3=8;
	while(1) 	/* **** Infinite cycle  ***** */
	{
		P4.7 = 0;
		/********************************************/
		/* Test for synchronous messages      */
		/********************************************/
		if(SEC_count > 0)
		{
			/* If entering in a new EC */

			SEC_count = 0;
			base2++;

			value2[0] = base;
			produce( 2,value2);
			base++;

			if(base2>=4)
			{
				value6[0] = base1;
				produce( 6,value6);
				base1++;
				base2=0;
			}
		}

       	#ifdef TEST_ASY
			/********************************************/
			/* Test for asynchronous messages           */
			/********************************************/
			if(AEC_count > AMS2-1)
			{

				AEC_count=0;

				while(AM_stop == FALSE)
				{
					U8051NOP
				}

				/* Produce async mesg AIS2 with a "period" of AMS2 ECs */
				value9[0]=base3;
				mesgid=AIS2;
				mesglen=ALS2;
				result=AM_produce(mesgid,mesglen,value9);
				if(result == AMOK)
				{
					// printf("\nAM_produce AMOK");
				}
				base3++;
			}

			if(dbgbufindex >= DBGBUFSIZE-10)
			{
				EA = 0;

				printf("\n\n Tempos de producao/transmissao de mensagens\n....");
				printf("\n Station 2 : Mesg ID=%d Len=%d mit=%d\n",AIS2,ALS2,AMS2);
				for(i=0;i<DBGBUFSIZE-10;i++)
				{
					printf("\n>>> SN:%d : T2B_Ticks:%d : TMH2:%d : TML2:%d : Tipo:%d",i,dbgbuf[i][0],dbgbuf[i][1],dbgbuf[i][2],dbgbuf[i][3]);
				}
				while(1);
				EA = 1;
				dbgbufindex =0;
			}
		#endif
	}
#endif

#ifdef STATION3

	base=3;
	base3=9;
	while(1) 	/* **** Infinite cycle  ***** */
	{
		P4.7 = 0;
		/********************************************/
		/* Test for synchronous messages      */
		/********************************************/
		if(SEC_count > 1)
		{
			/* If entering in a new EC */

			SEC_count = 0;

			value3[0] = base;
			produce( 3,value3);
			base++;

		}

       	#ifdef TEST_ASY
			/********************************************/
			/* Test for asynchronous messages           */
       	 	/********************************************/
			if(AEC_count > AMS3-1)
			{

				AEC_count=0;

				while(AM_stop == FALSE)
				{
					U8051NOP
				}

				/* Produce async mesg AIS3 with a "period" of AMS3 ECs */
				value9[0]=base3;
				mesgid=AIS3;
				mesglen=ALS3;
				result=AM_produce(mesgid,mesglen,value9);
				if(result == AMOK)
				{
					// printf("\nAM_produce AMOK");
				}
				base3++;
			}

            if(dbgbufindex >= DBGBUFSIZE-10)
			{
				EA = 0;

				printf("\n\n Tempos de producao/transmissao de mensagens\n....");
				printf("\n Station 3 : Mesg ID=%d Len=%d mit=%d\n",AIS3,ALS3,AMS3);
				for(i=0;i<DBGBUFSIZE-10;i++)
				{
					printf("\n>>> SN:%d : T2B_Ticks:%d : TMH2:%d : TML2:%d : Tipo:%d",i,dbgbuf[i][0],dbgbuf[i][1],dbgbuf[i][2],dbgbuf[i][3]);
				}
				while(1);
				EA = 1;
				dbgbufindex =0;
			}

		#endif
	}
#endif

#ifdef STATION4
	base=4;
	base3=10;
	while(1) 	/* **** Infinite cycle  ***** */
	{
		P4.7 = 0;
		/********************************************/
		/* Test for synchronous messages      */
		/********************************************/
		if(SEC_count > 0)
		{
			/* If entering in a new EC */

            SEC_count = 0;

			value4[0] = base;
			produce( 4,value4);
			base++;

		}

       	#ifdef TEST_ASY
			/********************************************/
			/* Test for asynchronous messages           */
			/********************************************/
			if(AEC_count > AMS4-1)
			{

				AEC_count=0;

				while(AM_stop == FALSE)
				{
					U8051NOP
				}

				/* Produce async mesg AIS4 with a "period" of AMS4 ECs */
				value9[0]=base3;
				mesgid=AIS4;
				mesglen=ALS4;
				result=AM_produce(mesgid,mesglen,value9);
				if(result == AMOK)
				{
					// printf("\nAM_produce AMOK");
				}
				base3++;
			}

			if(dbgbufindex >= DBGBUFSIZE-10)
			{
				EA = 0;

				printf("\n\n Tempos de producao/transmissao de mensagens\n....");
				printf("\n Station 4 : Mesg ID=%d Len=%d mit=%d\n",AIS4,ALS4,AMS4);
				for(i=0;i<DBGBUFSIZE-10;i++)
				{
					printf("\n>>> SN:%d : T2B_Ticks:%d : TMH2:%d : TML2:%d : Tipo:%d",i,dbgbuf[i][0],dbgbuf[i][1],dbgbuf[i][2],dbgbuf[i][3]);
				}
				while(1);
				EA = 1;
				dbgbufindex =0;
			}
		#endif
	}
#endif

#ifdef STATION5
	base3=11;
	while(1) 	/* **** Infinite cycle  ***** */
	{
		P4.7 = 0;
		/********************************************/
		/* Test for synchronous messages      */
		/********************************************/

		if(SEC_count > 0)
		{
			/* If entering in a new EC */
			SEC_count = 0;
		}

		/* DEBUG*/
		/*
		errcode = CAN_SR & 0x80; /* BUS Satus (1: Bus off; 0: Bus on) */

		if(errcode)
			printf("\nErr! BUS OFF");
		else
			printf("\nBUS ON");
		*/

		#ifdef TEST_ASY
			/********************************************/
			/* Test for asynchronous messages           */
			/********************************************/
			if(AEC_count > AMS5-1)
			{

				AEC_count=0;

				while(AM_stop == FALSE)
				{
					U8051NOP
				}

				/* Produce async mesg AIS5 with a "period" of AMS5 ECs */
				value9[0]=base3;
				mesgid=AIS5;
				mesglen=ALS5;
				result=AM_produce(mesgid,mesglen,value9);
				if(result == AMOK)
				{
					// printf("\nAM_produce AMOK");
				}
				base3++;
			}

			if(dbgbufindex >= DBGBUFSIZE-10)
			{
				EA = 0;

				printf("\n\n Tempos de producao/transmissao de mensagens\n....");
				printf("\n Station 5 : Mesg ID=%d Len=%d mit=%d\n",AIS5,ALS5,AMS5);
				for(i=0;i<DBGBUFSIZE-10;i++)
				{
					printf("\n>>> SN:%d : T2B_Ticks:%d : TMH2:%d : TML2:%d : Tipo:%d",i,dbgbuf[i][0],dbgbuf[i][1],dbgbuf[i][2],dbgbuf[i][3]);
				}
				while(1);
				EA = 1;
				dbgbufindex =0;
			}
		#endif
	}
#endif

#ifdef STATION6
	base3=12;
	while(1) 	/* **** Infinite cycle  ***** */
	{
		P4.7 = 0;
		/********************************************/
		/* Test for synchronous messages      */
		/********************************************/
		if(SEC_count > 0)
		{
			/* If entering in a new EC */
			SEC_count = 0;

		}

       	#ifdef TEST_ASY
			/********************************************/
			/* Test for asynchronous messages           */
			/********************************************/
			if(AEC_count > AMS6-1)
			{

				AEC_count=0;

				while(AM_stop == FALSE)
				{
					U8051NOP
				}

				/* Produce async mesg AIS6 with a "period" of AMS6 ECs */
				value9[0]=base3;
				mesgid=AIS6;
				mesglen=ALS6;
				result=AM_produce(mesgid,mesglen,value9);
				if(result == AMOK)
				{
					// printf("\nAM_produce AMOK");
				}
				base3++;
			}

			if(dbgbufindex >= DBGBUFSIZE-10)
			{
				EA = 0;

				printf("\n\n Tempos de producao/transmissao de mensagens\n....");
				printf("\n Station 6 : Mesg ID=%d Len=%d mit=%d\n",AIS6,ALS6,AMS6);
				for(i=0;i<DBGBUFSIZE-10;i++)
				{
					printf("\n>>> SN:%d : T2B_Ticks:%d : TMH2:%d : TML2:%d : Tipo:%d",i,dbgbuf[i][0],dbgbuf[i][1],dbgbuf[i][2],dbgbuf[i][3]);
				}
				while(1);
				EA = 1;
				dbgbufindex =0;
			}

		#endif
	}
#endif

} /* END - Main */

/*********************************
* ini_globals
*
* initialises global structures
**********************************/
void ini_globals(void)
{
	unsigned char i;


    /* Production buffer empty */
	StNProdMesg = 0;

    /* Variables' table empty - Synchronous Messages*/
    var_table.nvars=0;
    for( i=0; i < MAX_VAR; i++)
        var_table.vars[i].status = 0;


    /* Initialise timer 0, for signaling synch window beginning */
    timer_ini(  0,      /* Timer 0 */
		        0,      /* Timer   */
	        	0,      /* Modo = 0; int @112.5Hz (8.9ms) */
		                /* Modo = 1; int @14.06Hz (71ms)  */
		        0,      /* Controlo s pelo bit TR0 */
		        1,      /* Gera ints */
		        1,      /* Priority */
		        0);     /* Recarga; N/A */
               /* starts later, upon reception of Ec master message */

    EA=1;   /* Enable interrupts */
}

/************************************
* ini_CAN
*
* Inicializao do interface CAN
*  ( com interrupo )
*************************************/
void ini_CAN(void)
{

    /* Inicializao do interface CAN */
    /* Criado com 80592.xls */
    /* Baud Rate = 122.88 kHz(@11.0592MHz), Interface=250 */

    /* force Reset Request, accept CAN tx and rx ints, only */
    CANADR = CAN_CR;
    CANDAT = 0x63;  /* TM=0, S=RA=1, OIE=EIE=0, TIE=0, RIE=1, RR=1 */
    CANADR = CAN_CMR;
    CANDAT = 0xC0;   /* RX0A=RX1A=1, WUM=SLP=0 */

  

    CANADR = CAN_ACR | AutoInc;
    /*  CANDAT = (MST_MESG_ID>>3)|(DATA_MESG_ID>>3); / ACR - gen int upon rx of master/data messages only */
    /*  CANDAT = (MST_MESG_ID>>3)|(DATA_MESG_ID>>3)|0x7; / AMR- relevant bits (=0) */
    CANDAT = 0; /*ACR*/
    CANDAT = 0xff; /*AMR - all bits don't care, receive all messages */
    CANDAT = 0x44; /* BTR0 */
    CANDAT = 0xA4; /* BTR1 */
    CANDAT = 0x1A; /* OCR */
  
    /* terminate Reset Request and keep other bits as they were */
    CANADR = CAN_CR;
    CANDAT &= 0xFE; /* RR=0 */

    /* Interrupt Setup  */
    ES1=1;  /* Enable SIO1 (CAN) Interrupt          */
	PS1=1;  /* Priority Level of SIO1 (CAN) = high  */
	return;
  
}

/************************************
* ini_T2
*
* Initialize Timer2 in timer mode, with
*   8 bit overflow interrupt
*
*
*************************************/
void ini_T2(void)
{
	T2B_Ticks = 0;	/* Init counter of Timer 2 overflow ticks */

	IEN1 = 0x80;	/* bit 7       	>ET2=1:Enab. Timer 2 overflow interrupts		*/
					/* bit 6,5,4   	>ECM(2,1,0): T2 comparator interrupt enab/disab	*/
					/* bit 3,2,1,0 	>ECT(2,1,0): T2 capture interrupt enab/disab	*/

	TM2CON = 0x81;	/* bit 7	>T2IS1=1:16 bit overflow int enabled 	*/
					/* bit 6	>T2IS0=0:byte overflow int disabled 	*/
					/* bit 5	>T2ER=0:T2 external reset disabled		*/
					/* bit 4	>T2BO : Timer 2 byte overflow flag		*/
					/* bit 3,2	>T2Px : Timer T2 prescaler (0,0)->Clock Source */
					/* bit 1,0	>T2MSx : Timer 2 mode select 			*/
					/*				0,0 : Halted						*/
					/*			   >0,1	: T2 clock source = fosc/12		*/
					/*				1,0 : Test mode						*/
					/*				1,1 : T2 clock source=Pin T2		*/

	PT2=1;	/* T2 interrupt with high priority */

  /* This configuration generates a T2 16 bit overflow each (aprox) 256*256us		*/
  /* Time measurements have aprox. 1us resolution	*/

}


/*******************************************************************
* AM_ini
*
*  Initialization of Asynchronous Messaging System 
*    of FTT-CAN.
*  
*  Indexes for insertion and removal from buffer are initialized
*  
********************************************************************/
void AM_ini(void)
{
    unsigned char i;
    
    AM_buffer.AM_putind=0;
    AM_buffer.AM_getind=0;
    win_phase=ASY_WIN;  /* Current phase initialized to Asynchronous Window */

	/* Variables table empty - Asynchronous Messages*/
	am_var_table.nvars=0;
	for( i=0; i < AM_STAMAXVAR; i++)
		am_var_table.vars[i].status = 0;

    
}


/*********************************
* S1_int
*
* Rotina de servio ao interrupt CAN 
* (Serial port 1)
*
*   station interrupt service routine
*
*   interpretes master message data
*   produce all variables specified in the
*      master message that belong to this station
*   consumes all variables that belong to this station
*
*   globals used:
*     S1_i, S1_j , general indexes
*
****************************/

interrupt void S1_int(void)
{

    data unsigned char i;
    data unsigned char highid, interrupts;
    StaVarType *aux_var;
    AM_ConsStaVarType *am_aux_var;
    unsigned char *aux_pointer;


    /* Leitura do registo CANCON (read- int register) */
    interrupts = CANCON;
    
    /*******************************************/
    /* Do nothing if error or wakeup interrupt */
    /*******************************************/

    #ifdef DEVELOP
        if (interrupts & 0x04) {};     /* error - EI */    /* Disabled */
        if (interrupts & 0x10) {};     /* wake up - WUI */
    #endif
    
    /********************************************/
    /* Overrun error of reception buffer, do ?? */   
    /********************************************/
    if ( interrupts & 0x08 )     /* overrun - OI */
	{
	    /* CMR: clear overrun status COS */
	    CANADR = CAN_CMR;
		CANDAT &= 0xf8;  /* COS is read as 1, high bits must be kept */

        /* follow down to receive last message, forget previous one */
	    /* rx int. must be pending too */
    }

    /**************************************/
    /* CAN Rx INT : message just received */
    /**************************************/
    if( interrupts & 0x01 )     /* RI - rx int. */
    {
        /* Read the full frame of the master message into IntMesgBuf[]  */
        /* Use DMA (buffer is in data memory (internal)                 */
        /* Set internal RAM add in CANSTA                               */
        CANSTA = (unsigned char) IntMesgBuf;
        /* start DMA to receive full frame (start at CAN_RX) */
        CANADR = CAN_RX | DMA;

        /* Make time for DMA transfer (2 instr. cycle) */
        U8051NOP
        U8051NOP

        /* CMR: release RX buffer, RRB */
        CANADR = CAN_CMR;
        CANDAT &= 0xf4;  /* RRB is read as 1, high bits must be kept */
    
        /***************************/
        /* If Trigger Message, ... */
        /***************************/
        if( (IntMesgBuf[0] & 0xF0) == (MST_MESG_ID>>3) )
        {

        	/* Check if current phase is asynchronous window -> duplicated TM -> Discard message*/
        	if(win_phase == ASY_WIN)
        		return;
        	

			/* Get Asynch Window Duration                  */
			/* Set Timer 0 (Begin of synch window)         */

			/*Extract # of 500us slots nedded for synch. mesgs, from Master Message */
			smlen_500us = IntMesgBuf[2];
			/* Start timer0 to generate an Interrupt after Scan Window end         */
			/* Timer Int should occur at: 9.8ms(255) - time needed for synch mesgs */
			/*   - Timer0 Int And Can overhead - Time used by Master Message       */
			TH0=  smlen_500us * SLOT500US_TICKS+TIMINT_CAN_OVERH + EC_MASTER_TICKS + MAXAMVAR_TICKS + OVRHEAD_QUEU;
			TL0= 0;
			TR0= 1; /* Start Timer 0 */

			/* Set T2 compare register 0 to generate int during next trigger message*/
			/*   This way, there is no gap following the trig. mesg and priority 	*/
			/*   inversions concerning the first asynch mesg can be avoided.		*/

			P4.4=1; /* DEBUG */

			CML0=TML2;
			CMH0=TMH2;


			if( (unsigned int)CML0+EC_MESGWIN_T2tL>255)
				CMH0++;

			CML0+=EC_MESGWIN_T2tL;
			CMH0+=EC_MESGWIN_T2tH;


			/* Enable ints on T2 compare register 0*/
			IEN1 |= 0x10;   /* Disable T2 comparator 0 interrupts */
						/* bit 7       	>ET2		: T2 overflow interrupts				*/
						/* bit 6,5,4   	>ECM(2,1,0)	: T2 comparator interrupt enab/disab	*/
						/* bit 3,2,1,0 	>ECT(2,1,0)	: T2 capture interrupt enab/disab		*/

			P4.5 = 1;

			/* Set-up of status variables */
			win_phase = ASY_WIN;/* Current phase is asynchronous window */
			AM_stop = FALSE;    /* Signals that Asynchronus Messages are allowed */



			/* Save Trigger Message data in internal buffer IntMMBuf */
			for( S1_i = 0; S1_i < (IntMesgBuf[1] & 0xf)-1; S1_i++)
				IntMMBuf[S1_i] = IntMesgBuf[S1_i+3];


			/* Timing info BEGIN */
			/*   Store instant of Rx of trigger message*/
			//
			//A_TML2=TML2;
			//A_TMH2=TMH2;
			//A_T2B_Ticks = T2B_Ticks;
			//
			//dbgbuf[dbgbufindex][0]=A_T2B_Ticks;
			//dbgbuf[dbgbufindex][1]=A_TMH2;
			//dbgbuf[dbgbufindex][2]=A_TML2;
			//dbgbuf[dbgbufindex][3]=2;
			//dbgbufindex++;
			//
			/* Timing info - END */


			/* Increment EC_counters to trigger message production */
			SEC_count++;
			AEC_count++;

			P4.5 = 0;

			/****************************************************/
			/* Prepare production of synchronous messages       */
			/*      Fill IntTxBuf with first message            */
			/*      Update scan table variables & counters      */
			/****************************************************/

			/* Initialise EC message counters */
			NEcTxMesg= 0;
			ScanIndex= 0;
			NEcRxMesg= 0;

            /* Go through the station's production buffer to identify       */
            /*   vars to be produced in this EC by this station             */
            /* If any message is found, it will be placed in IntTxBuff to   */
            /*   allow DMA use in T0 int serv routine                       */
            for( S1_i = 0; S1_i < StNProdMesg; S1_i++)
            {
                /* Check if respective bit is set in the master message data */
                if( IntMMBuf[(StProdByteIndex[S1_i])] & (StProdBitMask[S1_i]) )
                { 
                    /* Bit set, produce this var */
                    /* build index to table */
                    S1_j = StProdByteIndex[S1_i] * 8;
                    switch( StProdBitMask[S1_i] )
                    {
                        case 0x01:
                            break;
                        case 0x02:
                            S1_j++;
                            break;
                        case 0x04:
                            S1_j += 2;
                            break;
                        case 0x08:
                            S1_j += 3;
                            break;
                        case 0x10:
                            S1_j += 4;
                            break;
                        case 0x20:
                            S1_j += 5;
                            break;
                        case 0x40:
                            S1_j += 6;
                            break;
                        case 0x80:
                            S1_j += 7;
                    }
                    aux_var = &(var_table.vars[S1_j]);

                    /* Copy message to IntTxBuf to allow use of DMA in T0 int serv routine */
                    IntTxBuf[0] = (aux_var->vid>>3) | (DATA_MESG_ID>>3); /* ID high byte */
                    /* code PROMPT bit in id (bit 6) */
                    if( (aux_var->status) & TX_NEW_DAT )
                        IntTxBuf[0] |= (TXNEWDAT_MASK>>3); 

                    /* reset the TX_NEW_DAT status */
                    aux_var->status &= ~TX_NEW_DAT;
                    IntTxBuf[1] = ((aux_var->vid)<<5) | aux_var->size; /* ID low byte + DLC */
                    if( (aux_var->status) & LASTBUF )
                    {   /* last data-buffer is data1, read from it */
                        for( S1_j=0; S1_j < aux_var->size; S1_j++)
                            IntTxBuf[S1_j+2] = aux_var->data1[S1_j];
                    }
                    else
                    {   /* last data-buffer is data0, read from it */
                        for( S1_j=0; S1_j < aux_var->size; S1_j++)
                            IntTxBuf[S1_j+2] = aux_var->data0[S1_j];
                    }

                    /* Save index to production buffer          */
                    /* Scanning goes on in T0 int serv. routine */
                    ScanIndex = S1_i;
                    NEcTxMesg = 1;
                    break;

                } /* If bit set in Master Message */
			}  /* For each variable */


        } /* IF Trigger Message */

        /**********************************/
        /* If Synchronous Data Message... */
        /**********************************/
        else if( (IntMesgBuf[0] & ~((DMESG_MASK|TXNEWDAT_MASK)>>3)) == (DATA_MESG_ID>>3) )
        {   /* Consume the data to the right slot in the table */
            
            /* Build low 6 bits of identifier */
            S1_i = ((IntMesgBuf[0]&0x07)<<3) | (IntMesgBuf[1]>>5);
            
            /* Use it as table index */
	        aux_var = &(var_table.vars[S1_i]);

	        if( aux_var->status & CONSUMER )    /* Station is a consumer of this message */
            {   
	            /* Save data in the right place */
                /* Check LASTBUF flag for data double-buffer control */
                if( (aux_var->status) & LASTBUF )
                {   /* last buffer written was data1, write to data0 */
                    for( S1_j = 0; S1_j < (IntMesgBuf[1] & 0xf); S1_j++)
                        *(aux_var->data0 + S1_j) = IntMesgBuf[S1_j+2];
                    /* update LASTBUF flag to 0 */
                    (aux_var->status) &= ~LASTBUF;
                }
                else
                {   /* Last buffer written was data0, write to data1 */
                    for( S1_j=0; S1_j < (IntMesgBuf[1] & 0xf); S1_j++)
                        *(aux_var->data1 + S1_j) = IntMesgBuf[S1_j+2];
                    /* update LASTBUF flag to 1 */
                    (aux_var->status) |= LASTBUF;
                }
                
                /* Save PROMPT bit */
                if( IntMesgBuf[0] & (TXNEWDAT_MASK>>3) )
                    (aux_var->status) |= TX_NEW_DAT;
                else
                    (aux_var->status) &= ~TX_NEW_DAT;
                
                /* Set RX_NEW_DAT bit */
                (aux_var->status) |= RX_NEW_DAT;
            } /* If Consumer */

        }
        
        /**********************************/
        /* If Asynchronous Data Message.. */
        /**********************************/
        else if( (IntMesgBuf[0] & ~((DMESG_MASK|TXNEWDAT_MASK)>>3)) == (AM_DATA_MESG_ID>>3) )
        {   /* Consume the data to the right slot in the table */
            
            /* Build low 6 bits of identifier */
            S1_i = ((IntMesgBuf[0]&0x07)<<3) | (IntMesgBuf[1]>>5);
            
            /* Scan Asynchronous Message Table */

            for(i = 0;i < am_var_table.nvars;i++)
            {
                
                if(am_var_table.vars[i].vid == S1_i)
                {
                    /* Use it as table index */
	                am_aux_var = &(am_var_table.vars[i]);

                    /* Save data in the right place */
                    /* Check LASTBUF flag for data double-buffer control */
                    if( (am_aux_var->status) & LASTBUF )
                    {   /* last buffer written was data1, write to data0 */
                        for( S1_j = 0; S1_j < (IntMesgBuf[1] & 0xf); S1_j++)
                            *(am_aux_var->data0 + S1_j) = IntMesgBuf[S1_j+2];
                        /* update LASTBUF flag to 0 */
                        (am_aux_var->status) &= ~LASTBUF;
                    }
                    else
                    {   /* Last buffer written was data0, write to data1 */
                        for( S1_j=0; S1_j < (IntMesgBuf[1] & 0xf); S1_j++)
                            *(am_aux_var->data1 + S1_j) = IntMesgBuf[S1_j+2];
                        /* update LASTBUF flag to 1 */
                        (am_aux_var->status) |= LASTBUF;
                    }
                
                    /* Save PROMPT bit */
                    if( IntMesgBuf[0] & (TXNEWDAT_MASK>>3) )
                        (am_aux_var->status) |= TX_NEW_DAT;
                    else
                        (am_aux_var->status) &= ~TX_NEW_DAT;
                    
                    /* Set RX_NEW_DAT bit */
                    (am_aux_var->status) |= RX_NEW_DAT;

                    break; /* Exit for */
                }
            }
        } /* Async. Data Message Received */

    } /* RX Int */


    /**********************************/
    /* CAN Tx INT : message just sent */
    /**********************************/
    if( interrupts & 0x02 )     /* tx int. - TI */
    {
        
        if(win_phase == SYN_WIN)
        {           
            /*****************************************************/
            /* Synchronous window - produce synchronous messages */
            /*****************************************************/

            /* Follow on with production of messages */
            /* Activate DMA to tx message in IntTxBuf */
            /* Set internal RAM add in CANSTA */
            CANSTA = (unsigned char) IntTxBuf;
            /* start DMA to tx full frame */
            CANADR = CAN_TX | DMA;
        
             /* Make time for DMA transfer (2 instr. cycle) */
            U8051NOP
            U8051NOP

            /* Set TR bit (Transmission Request) */
            CANADR = CAN_CMR;
            CANDAT &= 0xF1;    /* keep high 4 bits, TR is read as 1, reset others */

            /* Continue scanning the production buffer */
            for( S1_i = ScanIndex+1; S1_i < StNProdMesg; S1_i++)
            {
                /* Check if respective bit is set in the master message data */
                if( IntMMBuf[(StProdByteIndex[S1_i])] & (StProdBitMask[S1_i]) )
                {   /* bit set, produce this var */
                    /* build index to table */
                    S1_j = StProdByteIndex[S1_i] * 8;
                    switch( StProdBitMask[S1_i] )
                    {
                        case 0x01:
                            break;
                        case 0x02:
                            S1_j++;
                            break;
                        case 0x04:
                            S1_j += 2;
                            break;
                        case 0x08:
                            S1_j += 3;
                            break;
                        case 0x10:
                            S1_j += 4;
                            break;
                        case 0x20:
                            S1_j += 5;
                            break;
                        case 0x40:
                            S1_j += 6;
                            break;
                        case 0x80:
                            S1_j += 7;
                    }
        
                    /* Copy next message to IntTxBuf to allow use of DMA */  
                    /* this is done while previous is being tx, CPU-I/O overlapping */
                    aux_var = &(var_table.vars[S1_j]);
                    IntTxBuf[0] = (aux_var->vid>>3) | (DATA_MESG_ID>>3); /* ID high byte */
                    /* code TXNEWDAT bit in id (bit 6) */
                    if( (aux_var->status) & TX_NEW_DAT )
                        IntTxBuf[0] |= (TXNEWDAT_MASK>>3); 
        
                    /* reset the TX_NEW_DAT status */
                    aux_var->status &= ~TX_NEW_DAT;
                    IntTxBuf[1] = ((aux_var->vid)<<5) | aux_var->size; /* ID low byte + DLC */
        
                    if( (aux_var->status) & LASTBUF )
                    {   /* last data-buffer is data1, read from it */
                        for( S1_j=0; S1_j < aux_var->size; S1_j++)
                            IntTxBuf[S1_j+2] = aux_var->data1[S1_j];
                    }
                    else
                    {   /* last data-buffer is data0, read from it */
                        for( S1_j=0; S1_j < aux_var->size; S1_j++)
                            IntTxBuf[S1_j+2] = aux_var->data0[S1_j];
                    }
                
                    /* notice CAN tx ints remain enabled */
    
                    /* save index to production buffer */
                    /* scanning goes on in next CAN tx int  */
                    ScanIndex = S1_i;
                    /* update counter of produced messages */
                    NEcTxMesg++;
                    break;
                } /* if bit set in Master Message */
            } /* For each variable in Production Buffer */

            if( S1_i >= StNProdMesg )
            {
                /* Disable Tx Ints */
                CANADR = CAN_CR; 
                CANDAT = 0x62;  /* Bit 2:TIE=0 ; Bit 1:RIE=1*/    
            }
        }


		if(win_phase == ASY_WIN)
		{
			/*******************************************************/
			/* Asynchronous window - produce asynchronous messages */
			/*******************************************************/

			if( (CANSTA & 0x08) ) /* StatReg:Bit3-Tx Complet Stat(1-Last TX OK;0-Last TX Fail)*/
			{
				/* Last Tx request OK, so remove from AM buffer queue */
				AMgb_getmode=UPDATEINDEX;
				AM_getbuf(); /* Remove Message from AM Buffer */

            }


            AMS_TxReqPend=FALSE;    /* Reset flag of txmission on progress */

            if(AM_stop == FALSE)
            {
                /* Transmit next message in AM buffer (if any) */
                AMgb_getmode=GETDATA;
                if( AM_getbuf() == AMOK  )
                {
                    /* There is data in AM buffer waiting for transmission  */
                    /*   and the production window is not yet terminated    */

                    /* Activate DMA to tx message in AM_data */
                    /* Set internal RAM add in CANSTA */
                    CANSTA = (unsigned char) AMgb_data;
                    /* start DMA to tx full frame */
                    CANADR = CAN_TX | DMA;

                    /* Make time for DMA transfer (2 instr. cycle) */
                    U8051NOP
                    U8051NOP
                    
                    /* Set TR bit (Transmission Request) */
                    CANADR = CAN_CMR;
                    CANDAT &= 0xF1;     /* keep high 4 bits, TR is read as 1, reset others */

                    AMS_TxReqPend=TRUE; /* Signals AM Tx request */
                
                }
                else
                {
                    /* No more asynch. mesgs, so Stop Tx Ints. */
                    /* Disable Tx Ints */
                    CANADR = CAN_CR; 
                    CANDAT = 0x62;  /* Bit 2:TIE=0 ; Bit 1:RIE=1*/    

                }

            }
            else
            {
                /* End of async production window (AM_stop=TRUE)    */
                /* Disable Tx Ints                                  */
                CANADR = CAN_CR; 
                CANDAT = 0x62;  /* Bit 2:TIE=0 ; Bit 1:RIE=1*/    
            }
            
		} /* Asynchronous Window */

    } /* IF Rx INT */

  /* default return */
  return;
}


/********************************************************/
/*  T0_int                                              */
/*  Timer 0 interrupt service routine                   */
/*                                                      */
/*    Controls the duration of the synch window         */
/*                                                      */
/*     Started upon reception of master message         */
/*                                                      */
/*                                                      */
/********************************************************/
interrupt void T0_int(void)
{
    unsigned char i,highid;
    StaVarType *aux_var;


    /* Stops Timer 0 */
	TR0=0;

	if(AM_stop == FALSE && win_phase == ASY_WIN)
	{
		/********************************************************************/
		/* End of window of production of asynch mesgs in controlled mode   */
		/*  If the message in the buffer is already being txmitted, the     */
		/*    the following command will be irrelevant (see 80592 manual).  */
		/*  If the message is waiting for the bus it will be removed from   */
		/*    the buffer. Another try will be made in the following EC.     */
		/********************************************************************/

		P4.7=1;	/* DEBUG */
		/* Set timer to mark beginning of synch window */
		TH0= 255-MAXAMVAR_TICKS;
		TL0= 0;
		TR0= 1; /* Start Timer 0 */
		P4.7=0;	/* DEBUG */

        /* Signal end of production of asynch variables */
        AM_stop = TRUE;

        /* If messages are pending, remove them from TX buff. */
        /* Note that the Abort Txmission only has effect if Txmission not in progress */
		if(AMS_TxReqPend == TRUE)
		{
			/* Abort Tx of requested txmission */
			CANADR = CAN_CMR;
            CANDAT &= 0xF2;     /* Keep high 4 bits. AT is read as 1, reset others */

		}
		return;

	}

	if(AM_stop == TRUE && win_phase == ASY_WIN)
	{
		/********************************************************************/
		/* Start of synchronous window                                      */
		/* If one message is waiting for txmission in IntTxBuf,it is        */
		/*   Txmitted. Then the table of variables continue to be scanned.  */
		/* Following messages (if any) will be triggered by serial TxInt.   */
		/********************************************************************/

		win_phase=SYN_WIN;  /* Current phase is synchronous window */

        /* Test if CAN Tx buffer available. If not there is a timing violation. */
        /*  In this case, error recovery procedure should be triggered.         */
        if((CANSTA & 0x04) == 0) /* Stat.Reg.->Tx Buff Access(bit2) 1-Released; 0-Locked */
        {
            /* Synch. Window violation! Recovery action(s) follows. */

            /* Recovery action: exit. No synch. mesgs will be produced in this EC */
            /* This way the malfunction on this station will not be reflected at the bus level */
            /*   and if in the following EC all is OK, normal operation will be possible */
            return;     /* Note that Tx ints are disabled at this time */
        }

        /* If no message found, exit */
        if(NEcTxMesg == 0)
            return;
        
        /* Activate DMA to tx message in IntTxBuf */
        /* Set internal RAM add in CANSTA */
        CANSTA = (unsigned char) IntTxBuf;
        /* start DMA to tx full frame */
        CANADR = CAN_TX | DMA;

        /* Make time for DMA transfer (2 instr. cycle) */
        U8051NOP
        U8051NOP

        /* Set TR bit (Transmission Request) */
        CANADR = CAN_CMR;
        CANDAT &= 0xF1;     /* keep high 4 bits, TR is read as 1, reset others */

        /* Enable CAN Tx ints to produce following messages (if any) */
	    CANADR = CAN_CR; 
    	CANDAT = 0x66;     /* Bit 2:TIE=1; Bit 1:RIE=1 */

        /* Continue scanning the production buffer */
        for( S1_i = ScanIndex+1; S1_i < StNProdMesg; S1_i++)
        {
            /* Check if respective bit is set in the master message data */
            if( IntMMBuf[(StProdByteIndex[S1_i])] & (StProdBitMask[S1_i]) )
            {   /* bit set, produce this var */
                /* build index to table */
                S1_j = StProdByteIndex[S1_i] * 8;
                switch( StProdBitMask[S1_i] )
                {
                    case 0x01:
                        break;
                    case 0x02:
                        S1_j++;
                        break;
                    case 0x04:
                        S1_j += 2;
                        break;
                    case 0x08:
                        S1_j += 3;
                        break;
                    case 0x10:
                        S1_j += 4;
                        break;
                    case 0x20:
                        S1_j += 5;
                        break;
                    case 0x40:
                        S1_j += 6;
                        break;
                    case 0x80:
                        S1_j += 7;
                }

                /* Copy next message to IntTxBuf to allow use of DMA */  
                /* this is done while previous is being tx, CPU-I/O overlapping */
                aux_var = &(var_table.vars[S1_j]);
                IntTxBuf[0] = (aux_var->vid>>3) | (DATA_MESG_ID>>3); /* ID high byte */
                /* code TXNEWDAT bit in id (bit 6) */
                if( (aux_var->status) & TX_NEW_DAT )
                    IntTxBuf[0] |= (TXNEWDAT_MASK>>3); 
            
                /* reset the TX_NEW_DAT status */
                aux_var->status &= ~TX_NEW_DAT;
                IntTxBuf[1] = ((aux_var->vid)<<5) | aux_var->size; /* ID low byte + DLC */
                if( (aux_var->status) & LASTBUF )
                {   /* last data-buffer is data1, read from it */
                    for( S1_j=0; S1_j < aux_var->size; S1_j++)
                        IntTxBuf[S1_j+2] = aux_var->data1[S1_j];
                }
                else
                {   /* last data-buffer is data0, read from it */
                    for( S1_j=0; S1_j < aux_var->size; S1_j++)
                        IntTxBuf[S1_j+2] = aux_var->data0[S1_j];
                }

                /* Save index to production buffer */
                /* Scanning goes on in CAN tx int  */
                ScanIndex = S1_i;
                /* Update counter of produced messages */
                NEcTxMesg++;
                break;
            } /* if bit set in Master Message */
        } /* For each variable in Production Buffer */

        /* Test if any more message. If not, disable CAN Tx ints */
        if( S1_i >= StNProdMesg )
        {
            /* Production buffer fully scanned, disable CAN tx ints */
            /* Disable Tx Ints */
            CANADR = CAN_CR; 
            CANDAT = 0x62;  /* Bit 2:TIE=0 ; Bit 1:RIE=1*/    
		}
		return;
	}
	return;

}

/********************************************************/
/*  T2_int                                              */
/*  Timer 2 interrupt service routine                   */
/*                                                      */
/*                                                      */
/*                                                      */
/********************************************************/
interrupt void T2_int(void)
{

	/***************************************************/
	/* If interrupt generated by 16 bit timer overflow */
	/***************************************************/
	if(TM2IR & 0x80)
	{
		T2B_Ticks++;	/* Increment number of timer t2 overflow events */


		//if(TM2IR & 0x80) /* If 16 bit overflow int */
		//{
		//	if(Flag_T2==0)
		//	{
		//		Flag_T2 = 1;
		//		P4.1=1;
		//	}
		//	else
		//	{
		//		Flag_T2 = 0;
		//		P4.1=0;
		//	}
		//}

		//TM2CON &= 0xEF;	/* Clear T2BO bit : Timer 2 byte overflow int flag */
		TM2IR  &= 0x7F;	/* Clear T2OV bit : Timer 2 overflow (16bit) */

	}


	/****************************************************/
	/* If interrupt generated by compare register		*/
	/****************************************************/
	if(TM2IR & 0x10)
	{


		P4.4=0; /* DEGUG */

    	IEN1 &= 0xEF;   /* Disable T2 comparator 0 interrupts */
						/* bit 7       	>ET2		: T2 overflow interrupts				*/
						/* bit 6,5,4   	>ECM(2,1,0)	: T2 comparator interrupt enab/disab	*/
						/* bit 3,2,1,0 	>ECT(2,1,0)	: T2 capture interrupt enab/disab		*/

		TM2IR &= 0xEF;	/* Clear CMI0 bit : T2 compare register int Flag */

		/****************************************************************/
		/* This interrupt should occur during the Trigger Message 		*/
		/* Start production of asynchronous messages, while receiving 	*/
		/*	the Trig mesg to avoid idle time and priority inversion in 	*/
		/*  the first asynch mesg.										*/
		/****************************************************************/

		AMS_TxReqPend=FALSE;
		AMgb_getmode=GETDATA ; /* Next invocaton of AM_getbuf is to get next message */

		if( (AM_getbuf() == AMOK) && (CANSTA & 0x04) )
		{
			/* There is data in AM buffer waiting for transmission  */
			/*  and the TX buffer is free                           */
			/*    Stat.Reg.->Tx Buff Access(bit2) 1-Released; 0-Locked */

			/* Activate DMA to tx message in AM_data */
			/* Set internal RAM add in CANSTA */
			CANSTA = (unsigned char) AMgb_data;
			/* start DMA to tx full frame */
			CANADR = CAN_TX | DMA;

			/* Make time for DMA transfer (2 instr. cycle) */
			U8051NOP
			U8051NOP

			/* Set TR bit (Transmission Request) */
			CANADR = CAN_CMR;
			CANDAT &= 0xF1;     /* keep high 4 bits, TR is read as 1, reset others */

			/* Enable CAN Tx ints to produce following messages (if any) */
			CANADR = CAN_CR;
			CANDAT = 0x66;     /* Bit 2:TIE=1 ; Bit 1:RIE=1*/

			AMS_TxReqPend=TRUE; /* Signals a Tx request */

		}

		/* Set-up of status variables */
		win_phase = ASY_WIN;/* Current phase is asynchronous window */
		AM_stop = FALSE;    /* Signals that Asynchronus Messages are allowed */


	}

	return;

}


/***************************
*    StaTab_AddVar()
*
*   inserts a variable in the station variables' table
*   the table has a fixed size of MAX_VAR messages but
*   no more than STA_MAX_VAR messages for production
*   fills in auxiliar fields: byte_index and bit_mask
*
*   inputs:
*      - pointer to struct with properties of variable to add:
*         vid (6bit short identifier)
*         size (data size in bytes)
*         basic status (whether producer or consumer)
*      - (global) variables' table (StaVTabType var_table)
*      - (global) n. of variables to be produced by this station (StNProdMesg)
*      - (global) production buffers (StProdByteIndex , StProdBitMask)
*   outputs:
*      - (global) updated n. of vars to be produced by this station (StNProdMesg)
*      - (global) updated production buffers (StProdByteIndex , StProdBitMask)
*      - error code: 0 - OK
*                    -1 - table full (var_table.nvars = MAX_VAR)
*                    -2 - status not specified (aux_var->status = 0)
*                    -3 - vid index does not fit (aux_var->vid >= MAX_VAR)
*                    -4 - production buffer full (StNProdMesg = STA_MAX_VAR)
*
****************************/
signed char StaTab_AddVar(StaVarType* aux_var)
{
    unsigned char aux_bit,aux_byte;

    /* check if table is full, if yes - terminate with error */
    if( var_table.nvars == MAX_VAR )
	    return( -1 );

    /* check if input status==0, if yes - terminate with error */
    /* notice that a slot in the table is busy when status!=0 */
    if( aux_var->status == 0 )
	    return( -2 );

    /* check if vid fits within table, if no - terminate with error */
    /* notice that vid is used as an index to the messages' table */
    if( aux_var->vid >= MAX_VAR )
	    return( -3 );

    /* check if production buffers are full, if yes - terminate with error */
    if( (aux_var->status & PRODUCER) && (StNProdMesg == STA_MAX_VAR) )
	    return( -4 );

    /* complete the input structure, fields: byte_index and bit_mask */

    /* calculate respective byte and bit for codification in master message */
    /* get lower part of variable's identifier (6 bits stored in vid) */
    aux_bit = aux_var->vid;

    /* determine in which byte is the respective bit */
    aux_byte = 0;
    while( aux_bit > 7 )
	{
	    aux_byte++;
    	/* build bit mask for that byte */
	    aux_bit -= 8;
	}
    /* set bit_mask and byte_index in the var data structure */
    aux_var->byte_index = aux_byte;
    aux_var->bit_mask = 1;
    if( aux_bit > 0 )
        (aux_var->bit_mask) <<= aux_bit;


    /* insert var in table, slot is signalled busy by status!=0 */
    var_table.vars[aux_var->vid] = *aux_var;

    /* if var is to be produced, adjust production buffer */
    if( aux_var->status & PRODUCER )
    {
        StProdByteIndex[StNProdMesg] = aux_var->byte_index;
        StProdBitMask[StNProdMesg] = aux_var->bit_mask;
        StNProdMesg++;
    }

    /* adjust number of variables in table */
    var_table.nvars++;

    return( 0 );
}

/***************************
*   VarStatus()
*
*   used in stations
*   returns variable's status byte
*   the input vid is used as an index to the global table
*
*   inputs:
*      - reduced (low 6 bits) vid
*      - global variables' table (StaVTabType var_table)
*   outputs:
*      - status byte (except bit 7) (see above)
*      - error code: 
*                    -1 - var not in table
*
**************************/
signed char VarStatus(unsigned char VarID)
{
    unsigned char i;
    signed char ret_val;

    /* access var table with VarID as index */
    /* if does not exist in table (status==0) return (-1) */
    /* else, return full status byte (except bit 7 - because of sign) */

    if( (var_table.vars[VarID]).status != 0 )
        /* VarID exists in the local table, return status byte */
        ret_val = ((var_table.vars[VarID]).status & 0x7f);
    else
        ret_val = -1;


    return( ret_val );
}


/***************************************************************************
*   Produce()
*
*   used in stations
*   copies a variable's value into the value buffer in the Sta. Var. Table
*   checks wether station is a producer of this var
*   returns the promptness status
*   the input vid is used as an index to the global table
*
*   inputs:
*      - reduced (low 6 bits) vid
*      - pointer to the value to produce
*      - (global) variables' table (StaVTabType var_table)
*   outputs:
*      - promptness status of the variable (=0,1) (see above)
*      - error code: 
*                    -1 - var not in table
*                    -2 - station is not producer
*
***************************************************************************/
signed char produce (unsigned char VarID, unsigned char *VarValue)
{
    unsigned char i, datasize;
    signed char ret_val;
    unsigned char *aux_pointer;
    StaVarType *aux_var;

    /* access var table with VarID as index */
    /* if not found return (-1) */
    /* else, if not producer return (-2) */
    /* else (producer) */


    aux_var = &(var_table.vars[VarID]);

    /* check whether VarID exists in table (status != 0) */
    if( aux_var->status != 0 )
    { /* VarID exists in the local table, check whether station is producer */
        if( (aux_var->status) & PRODUCER )
        {   /* station is producer, produce this var */
            /* read var size and transfer value into double-buffer */
            datasize = aux_var->size;
            if( (aux_var->status) & LASTBUF )
            { /* last buffer written was data1, write to data0 */
                for(i=0; i < datasize; i++)
                    *(aux_var->data0+i) = *(VarValue+i);
                /* disable ints to avoid corruption of the PROMPT bit status */
                EA= 0;

                /* save previous TX_NEW_DAT bit */
                ret_val = (aux_var->status) & TX_NEW_DAT;
                /* set the TX_NEW_DAT bit */
                (aux_var->status) |= TX_NEW_DAT;
                /* update LASTBUF flag to 0 */
                (aux_var->status) &= ~LASTBUF;
                /* enable ints again */
                EA= 1;
            }
            else
            {   /* last buffer written was data0, write to data1 */
                for(i=0; i < datasize; i++)
                    *(aux_var->data1+i) = *(VarValue+i);
                /* disable ints to avoid corruption of the TX_NEW_DAT bit status */
                EA= 0;
                /* save previous TX_NEW_DAT bit */
                ret_val = (aux_var->status) & TX_NEW_DAT;
                /* set the TX_NEW_DAT bit and update LASTBUF flag to 1 */
                (aux_var->status) |= (TX_NEW_DAT | LASTBUF);
                /* enable ints again */
                EA= 1;
            }
        }
        else  /* station is not a producer of this var */
            ret_val = -2;
    }
    else  /* bit_mask == 0, VarID does not exist in this station */
        ret_val = -1;


    return (ret_val);
}

/***************************
*   Consume()
*
*   used in stations
*   copies a variable's value from the value buffer in the Sta. Var. Table
*   checks wether station is a consumer of this var
*   returns the promptness and refreshness status
*   the input vid is used as an index to the global table
*
*   inputs:
*      - reduced (low 6 bits) vid
*      - pointer where to copy the value to
*      - global variables' table (StaVTabType var_table)
*   outputs:
*      - promptness and refreshness status of the variable (=0,1,2,3) (see above)
*      - error code: 
*                    -1 - var not in table
*                    -2 - station is not consumer
*
****************************/

signed char consume (unsigned char VarID, unsigned char *VarValue)
{
    unsigned char i, ret_val, interrupts, datasize;
    StaVarType *aux_var;

    /* access var table with VarID as index */
    /* if not found return (-1) */
    /* else, if not consumer return (-2) */
    /* else (consumer) */


    aux_var = &(var_table.vars[VarID]);
 
    /* check whether VarID exists in table (status != 0) */
    if( aux_var->status != 0 )
    {   /* VarID exists in the local table, check whether station is consumer */
        if( (aux_var->status) & CONSUMER )
        {   /* consume this var */
            /* read var size */
            datasize=aux_var->size;
            /* disable ints to avoid corruption of the RX_NEW_DAT bit status */
            /*     and to assure correct transfer of all data bytes  */
            EA=0;
            /* save previous RX_NEW_DAT and TX_NEW_DAT bits */
            ret_val = (aux_var->status) & (RX_NEW_DAT|TX_NEW_DAT);
            /* reset the RX_NEW_DAT bit */
            (aux_var->status) &= ~RX_NEW_DAT;
            /* check which buffer must be read from */
            if( (aux_var->status) & LASTBUF )
            {   /* last buffer written was data1, read from it */
                /* enable ints again */
                EA=1;
                /* transfer value into buffer (i no longer needed in previous 'for') */
                for(i=0; i < datasize; i++)
                    VarValue[i] = aux_var->data1[i];
            }
            else
            {   /* last buffer written was data0, read from it */
                /* enable ints again, */
                EA=1;
                /* transfer value into buffer (i no longer needed in previous 'for') */
                for(i=0; i < datasize; i++)
                    VarValue[i] = aux_var->data0[i];
            }
        }
        else   /* station is not a consumer of this var */
            ret_val = -2;
    }
    else  /* this var does not exist in the global table */
        ret_val = -1;

    return (ret_val);
}


/****************************************************************************
*    AM_StaTab_AddVar()
*
*   Inserts a variable in the station asynchronous variables table
*   the table has a fixed size of AM_STAMAXVAR messages. Only variables
*   that are consumed in the station need to be stored.
*
*   inputs:
*      - pointer to struct with properties of variable to add:
*         vid (6bit short identifier)
*         size (data size in bytes)
*      - (global) variables' table (AM_ConsStaVTabType am_var_table)
*
*   outputs:
*      - error code: 0 - OK
*                    -1 - table full (am_var_table.nvars = AM_STAMAXVAR)
*                    -3 - invalid ID (> 6 bits <=> > 63d)
*
****************************************************************************/
signed char AM_StaTab_AddVar(AM_ConsStaVarType* aux_var)
{


    /* Check if table is full. If yes,terminate with error */
    if( am_var_table.nvars == AM_STAMAXVAR )
	    return( -1 );

    /* Check if vid is valid. If no, terminate with error */
    if( aux_var->vid > DMESG_MASK )
	    return( -3 );

    /* Set Status as Consumer */
    aux_var->status = CONSUMER;

    /* Insert var in table, slot is signalled busy by status!=0 */
    am_var_table.vars[am_var_table.nvars] = *aux_var;

    /* Adjust number of variables in table */
    am_var_table.nvars++;


    return( 0 );
}

/***************************
*   AM_VarStatus()
*
*   Used in stations, returns variable's status byte
*   of the variable vid
*
*   inputs:
*      - reduced (low 6 bits) vid
*      - global consumer asynchronous variables' table (AM_ConsStaVTabType am_var_table)
*
*   outputs:
*      - status byte 
*      - error code: 
*           -1 - var not in table
*
**************************/
signed char AM_VarStatus(unsigned char VarID)
{
    unsigned char i;
    signed char ret_val;

    ret_val = -1;

    /* If VarID invalid, exit with error */
    if(VarID > DMESG_MASK)
        return( ret_val );
     
    /* Scan Table of Async messages to find VarID */
    for(i=0;i<am_var_table.nvars;i++)
    {
        if(am_var_table.vars[i].vid == VarID)
        {
            ret_val = am_var_table.vars[i].status;
            break;            
        }
        
    }

    return( ret_val );
}

/*********************************************************************************
*   AM_Consume()
*
*   Used in stations
*   - Copies a variable's value from the value buffer in the AM Sta. Var. Table
*   - Checks if station is a consumer of this var
*   - Returns the promptness and refreshness status
*
*   inputs:
*      - reduced (low 6 bits) vid
*      - pointer where to copy the value to
*      - global variables' table (AM_ConsStaVTabType am_var_table)
*   outputs:
*      - promptness and refreshness status of the variable (=0,1,2,3) (see above)
*      - error code: 
*                    < 0 -  AMIID - var not in table;
*                    >= 0 - Status;
*
**********************************************************************************/

signed char AM_consume (unsigned char VarID, unsigned char *VarValue)
{
    unsigned char i,j, datasize;
    AM_ConsStaVarType *aux_var;
    signed char ret_val;


    ret_val = AMIID;

    /* If VarID invalid, exit with error */
    if(VarID > DMESG_MASK)
        return( ret_val );
     
    /* Scan Table of Async messages to find VarID */
    for(j=0;j<am_var_table.nvars;j++)
    {
        if(am_var_table.vars[j].vid == VarID)
        {
            /* Set pointer to var */
            aux_var = &(am_var_table.vars[j]);

            /* Get Status & DataSize */
            ret_val = aux_var->status;
            datasize = aux_var->size;

            /* Disable Ints to avoid data corruption */
            EA=0;   

            /* Save previous RX_NEW_DAT and TX_NEW_DAT bits */
            ret_val = (aux_var->status) & (RX_NEW_DAT|TX_NEW_DAT);
            /* Reset the RX_NEW_DAT bit */
            (aux_var->status) &= ~RX_NEW_DAT;
            
            /* check which buffer must be read from */
            if( (aux_var->status) & LASTBUF )
            {   /* Last buffer written was data1, read from it */
                
                EA=1;   /* Enable ints again */
                for(i=0; i < datasize; i++) /* Transfer value into buffer */

                    VarValue[i] = aux_var->data1[i];
                break;
            }
            else
            {   /* last buffer written was data0, read from it */
                
                EA=1;   /* Enable ints again, */
                for(i=0; i < datasize; i++)     /* Transfer value into buffer */
                    VarValue[i] = aux_var->data0[i];
                break;
            }
        }
    }
    return (ret_val);
}




/************************************************************
*   AM_produce()
*
*   Used in stations.
*   Copies a variable's value into the asynchronous Txmit buffer.
*   Checks wether the Txmit buffer is full.
*   
*   Inputs:
*       - reduced (low 6 bits) AM_msgid
*       - number of data bytes of the message
*       - pointer to the value to produce
*       
*   Outputs:
*       - error code: 
*           AMOK    -   success
*           AMBF    -   Buffer full
*           AMIID   -   Invalid Msg ID
*           AMIML   -   Invalid Msg length
*
**************************************************************/
signed char AM_produce (unsigned char AM_msgid, unsigned char AM_msglen, unsigned char *AM_msgdata)
{
    unsigned char i;
    unsigned char TMP_putind;
	unsigned char msg_head[2];



    /* Preliminary tests */
    if( (AM_msgid & !(0x3F)) != 0)     /* Message ID must be up to 6 bits */
        return(AMIID);

    if( AM_msglen > 8 )            /* Message length must be up to 8 bytes */
        return(AMIML);

	P4.2=1;

    /* Build message header */
    msg_head[0] = ( AM_msgid >>3 ) | ( AM_DATA_MESG_ID >> 3 ); /* ID high byte = AM_DATA_MESG_ID + MesgID */
    msg_head[0] |= (TXNEWDAT_MASK>>3);                      /* code TXNEWDAT bit in id (bit 6) */
    msg_head[1] = ((AM_msgid)<<5) | AM_msglen;              /* ID low byte + DLC */

	//EA=0;
	/* Test if buffer full */
	TMP_putind = AM_buffer.AM_putind + 1;
	if(TMP_putind == AM_BUF_LEN)                            /* If index overflows, restart from zero */
		TMP_putind = 0;
	if(TMP_putind == AM_buffer.AM_getind)                   /* If getind = putind then buffer full */
		return(AMBF);

    /* Store message in buffer */
    AM_buffer.AM_vars[AM_buffer.AM_putind].AM_data[0]=msg_head[0];
    AM_buffer.AM_vars[AM_buffer.AM_putind].AM_data[1]=msg_head[1];

    for(i=0;i<AM_msglen;i++)
        AM_buffer.AM_vars[AM_buffer.AM_putind].AM_data[i+2]=*(AM_msgdata+i);

    /* Update putind index value */
	AM_buffer.AM_putind=TMP_putind;
	//EA=1;

	P4.2=0;
	if(FMeasure == 1)
	{
		/* Timing info BEGIN */
		/*   Store instant of Txmission of asynch message*/

		//EA=0;
		A_TML2=TML2;
		A_TMH2=TMH2;
		A_T2B_Ticks = T2B_Ticks;
		//EA=1;
		dbgbuf[dbgbufindex][0]=A_T2B_Ticks;
		dbgbuf[dbgbufindex][1]=A_TMH2;
		dbgbuf[dbgbufindex][2]=A_TML2;
		dbgbuf[dbgbufindex][3]=1;
		dbgbufindex++;

		/* Timing info - END */
	}

    return(AMOK);
}

/*******************************************************************
* AM_getbuf
*
*  Used in Stations
*  Gets next row of the AM Txmit buffer.
*  Checks if AM Txmit buffer is empty
*
*  Inputs:
*         - AMgb_getmode
*               GETDATA - Get next message in buffer (if any) but do not update indexes
*               UPDATEINDEX - Update indexes correspondingly to removal of one message
*
*  Outputs:
*         - AMgb_buflen : Number of data bytes
*         - AMgb_data : Vector with data bytes
*
*   **** Note: Input and Output parameters via global vars to improve performance
*
*  Return:
*         AMOK - Sucess
*         AMEMPTY - Txmit Buffer empty
*  
********************************************************************/
signed char AM_getbuf( /* unsigned char getmode,unsigned char *AM_buflen, unsigned char *AM_data */)
{
        
    /*******************************/
    /* Check if any data in buffer */
    if(AM_buffer.AM_putind == AM_buffer.AM_getind)
        return(AMEMPTY);
   
    /************************/
    /* Get data from buffer */
    if(AMgb_getmode == GETDATA)
	{
		AMgb_aux_var=AM_buffer.AM_vars[AM_buffer.AM_getind].AM_data;
        AMgb_buflen = ((*(AMgb_aux_var+1)) & 0x0F) + 2; /* Two header bytes plus data bytes*/

        /* Copy from buffer to Output var */
        for(AMgb_i=0;AMgb_i < AMgb_buflen;AMgb_i++)
			*(AMgb_data+AMgb_i)=*(AMgb_aux_var+AMgb_i);

        return(AMOK);
    }

    /************************/
    /* Update getind index */
    if(AMgb_getmode == UPDATEINDEX)
    {
        
        AM_buffer.AM_getind++;
        if(AM_buffer.AM_getind==AM_BUF_LEN)
			AM_buffer.AM_getind=0;

		if(FMeasure == 1)
		{
			/* Timing info BEGIN */
			/*   Store instant of Txmission of asynch message*/

			A_TML2=TML2;
			A_TMH2=TMH2;
			A_T2B_Ticks = T2B_Ticks;
			dbgbuf[dbgbufindex][0]=A_T2B_Ticks;
			dbgbuf[dbgbufindex][1]=A_TMH2;
			dbgbuf[dbgbufindex][2]=A_TML2;
			dbgbuf[dbgbufindex][3]=0;
			dbgbufindex++;

			/* Timing info - END */
		}


        return(AMOK);    
    }
    
    return(AMOK);  /* Returns OK */
}
